@@ -1,4 +1,4 @@
-/*      $Id: wmtweaks_plugin.c 22775 2006-08-15 13:54:19Z olivier $
+/*      $Id: wmtweaks_plugin.c 25182 2007-03-17 09:26:51Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -55,6 +55,7 @@ static void run_dialog (McsPlugin * mcs_plugin);
 
 static gboolean is_running = FALSE;
 
+static gboolean borderless_maximize = TRUE;
 static gboolean cycle_minimum = TRUE;
 static gboolean cycle_hidden = TRUE;
 static gboolean cycle_workspaces = FALSE;
@@ -66,7 +67,7 @@ static gboolean raise_with_any_button = FALSE;
 static gboolean restore_on_move = TRUE;
 static gboolean scroll_workspaces = TRUE;
 static gboolean snap_resist = FALSE;
-static gboolean toggle_workspaces = TRUE;
+static gboolean toggle_workspaces = FALSE;
 static gboolean unredirect_overlays = FALSE;
 static gboolean use_compositing = FALSE;
 static gboolean wrap_layout = FALSE;
@@ -81,6 +82,7 @@ static int frame_opacity = 100;
 
 static char *easy_click = "Alt";
 /* 
+    "Xfwm/BorderlessMaximize"
     "Xfwm/CycleHidden"
     "Xfwm/CycleMinimum"
     "Xfwm/CycleWorkspaces"
@@ -458,6 +460,13 @@ create_dialog (McsPlugin * mcs_plugin)
     gtk_box_pack_start (GTK_BOX (vbox), check_button, FALSE, TRUE, 0);
     gtk_widget_show (check_button);
 
+    check_button =
+        create_gboolean_button (mcs_plugin,
+        _("Hide frame of windows when maximized"), "Xfwm/BorderlessMaximize",
+        &borderless_maximize);
+    gtk_box_pack_start (GTK_BOX (vbox), check_button, FALSE, TRUE, 0);
+    gtk_widget_show (check_button);
+
     check_button =
         create_gboolean_button (mcs_plugin,
         _("Restore original size of maximized windows when moving"), "Xfwm/RestoreOnMove",
@@ -730,6 +739,7 @@ xfwm4_create_channel (McsPlugin * mcs_plugin)
     }
     g_free (rcfile);
 
+    init_gboolean_setting (mcs_plugin, "Xfwm/BorderlessMaximize", &borderless_maximize);
     init_gboolean_setting (mcs_plugin, "Xfwm/CycleMinimum", &cycle_minimum);
     init_gboolean_setting (mcs_plugin, "Xfwm/CycleHidden", &cycle_hidden);
     init_gboolean_setting (mcs_plugin, "Xfwm/CycleWorkspaces", &cycle_workspaces);
