@@ -1,30 +1,28 @@
-/*      $Id: display.c 22296 2006-07-08 16:57:14Z olivier $
- 
+/*      $Id: display.c 25371 2007-04-02 19:32:01Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#include <sys/time.h>
-#include <time.h>
-
+#include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
@@ -33,7 +31,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 #ifdef HAVE_RENDER
 #include <X11/extensions/Xrender.h>
 #endif
@@ -48,6 +46,14 @@
 #define MAX_HOSTNAME_LENGTH 32
 #endif /* MAX_HOSTNAME_LENGTH */
 
+#ifndef CURSOR_ROOT
+#define CURSOR_ROOT XC_left_ptr
+#endif
+
+#ifndef CURSOR_MOVE
+#define CURSOR_MOVE XC_fleur
+#endif
+
 static int
 handleXError (Display * dpy, XErrorEvent * err)
 {
@@ -55,9 +61,9 @@ handleXError (Display * dpy, XErrorEvent * err)
     char buf[64];
 
     XGetErrorText (dpy, err->error_code, buf, 63);
-    g_print ("XError: %s\n", buf);                                                  
-    g_print ("==>  XID 0x%lx, Request %d, Error %d <==\n", 
-              err->resourceid, err->request_code, err->error_code); 
+    g_print ("XError: %s\n", buf);                                    
+    g_print ("==>  XID 0x%lx, Request %d, Error %d <==\n",
+              err->resourceid, err->request_code, err->error_code);
 #endif
     return 0;
 }
@@ -122,6 +128,8 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_WM_STATE_STICKY",
         "_NET_WM_STRUT",
         "_NET_WM_STRUT_PARTIAL",
+        "_NET_WM_SYNC_REQUEST",
+        "_NET_WM_SYNC_REQUEST_COUNTER",
         "_NET_WM_USER_TIME",
         "_NET_WM_WINDOW_TYPE",
         "_NET_WM_WINDOW_TYPE_DESKTOP",
@@ -158,14 +166,15 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "WM_TRANSIENT_FOR",
         "WM_WINDOW_ROLE",
         "XFWM4_COMPOSITING_MANAGER",
+        "XFWM4_TIMESTAMP_PROP",
         "_XROOTPMAP_ID",
         "_XSETROOT_ID"
     };
-    
-    g_assert (NB_ATOMS == G_N_ELEMENTS (atom_names));
-    return (XInternAtoms (display_info->dpy, 
-                          (char **) atom_names, 
-                          NB_ATOMS,
+
+    g_assert (ATOM_COUNT == G_N_ELEMENTS (atom_names));
+    return (XInternAtoms (display_info->dpy,
+                          (char **) atom_names,
+                          ATOM_COUNT,
                           FALSE, display_info->atoms) != 0);
 }
 
@@ -173,6 +182,7 @@ DisplayInfo *
 myDisplayInit (GdkDisplay *gdisplay)
 {
     DisplayInfo *display;
+    int major, minor;
     int dummy;
 
     display = g_new0 (DisplayInfo, 1);
@@ -193,11 +203,18 @@ myDisplayInit (GdkDisplay *gdisplay)
     }
 
     /* Test XShape extension support */
-    if (XShapeQueryExtension (display->dpy, 
-                              &display->shape_event_base, 
+    major = 0;
+    minor = 0;
+    display->shape_version = 0;
+    if (XShapeQueryExtension (display->dpy,
+                              &display->shape_event_base,
                               &dummy))
     {
         display->have_shape = TRUE;
+        if (XShapeQueryVersion (display->dpy, &major, &minor))
+        {
+            display->shape_version = major * 1000 + minor;
+        }
     }
     else
     {
@@ -206,6 +223,33 @@ myDisplayInit (GdkDisplay *gdisplay)
         display->shape_event_base = 0;
     }
 
+#ifdef HAVE_XSYNC
+    display->have_xsync = FALSE;
+    
+    display->xsync_error_base = 0;
+    display->xsync_event_base = 0;
+
+    major = SYNC_MAJOR_VERSION;
+    minor = SYNC_MINOR_VERSION;
+    
+    if (XSyncQueryExtension (display->dpy,
+                              &display->xsync_event_base,
+                              &display->xsync_error_base)
+         && XSyncInitialize (display->dpy, 
+                             &major, 
+                             &minor))
+    {
+        display->have_xsync = TRUE;
+    }
+    else
+    {
+        g_warning ("The display does not support the XSync extension.");
+        display->have_xsync = FALSE;
+        display->xsync_event_base = 0;
+        display->xsync_error_base = 0;
+    }
+#endif /* HAVE_XSYNC */
+
 #ifdef HAVE_RENDER
     if (XRenderQueryExtension (display->dpy,
                                &display->render_event_base,
@@ -220,13 +264,13 @@ myDisplayInit (GdkDisplay *gdisplay)
         display->render_event_base = 0;
         display->render_error_base = 0;
     }
-#else
+#else  /* HAVE_RENDER */
     display->have_render = FALSE;
-#endif
+#endif /* HAVE_RENDER */
 
 #ifdef HAVE_RANDR
     if (XRRQueryExtension (display->dpy,
-                            &display->xrandr_event_base, 
+                            &display->xrandr_event_base,
                             &display->xrandr_error_base))
     {
         display->have_xrandr = TRUE;
@@ -238,15 +282,15 @@ myDisplayInit (GdkDisplay *gdisplay)
         display->xrandr_event_base = 0;
         display->xrandr_error_base = 0;
     }
-#else
+#else  /* HAVE_RANDR */
     display->have_xrandr = FALSE;
-#endif
+#endif /* HAVE_RANDR */
 
-    display->root_cursor = 
-        XCreateFontCursor (display->dpy, XC_left_ptr);
-    display->move_cursor = 
-        XCreateFontCursor (display->dpy, XC_fleur);
-    display->busy_cursor = 
+    display->root_cursor =
+        XCreateFontCursor (display->dpy, CURSOR_ROOT);
+    display->move_cursor =
+        XCreateFontCursor (display->dpy, CURSOR_MOVE);
+    display->busy_cursor =
         cursorCreateSpinning (display->dpy);
     display->resize_cursor[CORNER_TOP_LEFT] =
         XCreateFontCursor (display->dpy, XC_top_left_corner);
@@ -256,13 +300,13 @@ myDisplayInit (GdkDisplay *gdisplay)
         XCreateFontCursor (display->dpy, XC_bottom_left_corner);
     display->resize_cursor[CORNER_BOTTOM_RIGHT] =
         XCreateFontCursor (display->dpy, XC_bottom_right_corner);
-    display->resize_cursor[4 + SIDE_LEFT] = 
+    display->resize_cursor[CORNER_COUNT + SIDE_LEFT] =
         XCreateFontCursor (display->dpy, XC_left_side);
-    display->resize_cursor[4 + SIDE_RIGHT] = 
+    display->resize_cursor[CORNER_COUNT + SIDE_RIGHT] =
         XCreateFontCursor (display->dpy, XC_right_side);
-    display->resize_cursor[4 + SIDE_TOP] = 
+    display->resize_cursor[CORNER_COUNT + SIDE_TOP] =
         XCreateFontCursor (display->dpy, XC_top_side);
-    display->resize_cursor[4 + SIDE_BOTTOM] = 
+    display->resize_cursor[CORNER_COUNT + SIDE_BOTTOM] =
         XCreateFontCursor (display->dpy, XC_bottom_side);
 
     display->xfilter = NULL;
@@ -279,7 +323,7 @@ myDisplayInit (GdkDisplay *gdisplay)
         g_free (display->hostname);
         display->hostname = NULL;
     }
-    
+
     compositorInitDisplay (display);
 
     return display;
@@ -296,7 +340,7 @@ myDisplayClose (DisplayInfo *display)
     display->move_cursor = None;
     XFreeCursor (display->dpy, display->root_cursor);
     display->root_cursor = None;
-    
+
     if (display->hostname)
     {
         g_free (display->hostname);
@@ -308,10 +352,10 @@ myDisplayClose (DisplayInfo *display)
         XFreeCursor (display->dpy, display->resize_cursor[i]);
         display->resize_cursor[i] = None;
     }
-    
+
     g_slist_free (display->clients);
     display->clients = NULL;
-    
+
     g_slist_free (display->screens);
     display->screens = NULL;
 
@@ -322,7 +366,7 @@ gboolean
 myDisplayHaveShape (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, FALSE);
-    
+
     return (display->have_shape);
 }
 
@@ -330,35 +374,35 @@ gboolean
 myDisplayHaveRender (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, FALSE);
-    
+
     return (display->have_render);
 }
 
-Cursor 
+Cursor
 myDisplayGetCursorBusy (DisplayInfo *display)
 {
     g_return_val_if_fail (display, None);
-    
+
     return display->busy_cursor;
 }
 
-Cursor 
+Cursor
 myDisplayGetCursorMove  (DisplayInfo *display)
 {
     g_return_val_if_fail (display, None);
-    
+
     return display->move_cursor;
 }
 
-Cursor 
+Cursor
 myDisplayGetCursorRoot (DisplayInfo *display)
 {
     g_return_val_if_fail (display, None);
-    
+
     return display->root_cursor;
 }
 
-Cursor 
+Cursor
 myDisplayGetCursorResize (DisplayInfo *display, guint index)
 {
     g_return_val_if_fail (display, None);
@@ -372,7 +416,7 @@ void
 myDisplayGrabServer (DisplayInfo *display)
 {
     g_return_if_fail (display);
-    
+
     DBG ("entering myDisplayGrabServer");
     if (display->xgrabcount == 0)
     {
@@ -386,6 +430,8 @@ myDisplayGrabServer (DisplayInfo *display)
 void
 myDisplayUngrabServer (DisplayInfo *display)
 {
+    g_return_if_fail (display);
+
     DBG ("entering myDisplayUngrabServer");
     display->xgrabcount = display->xgrabcount - 1;
     if (display->xgrabcount < 0)       /* should never happen */
@@ -401,7 +447,7 @@ myDisplayUngrabServer (DisplayInfo *display)
     DBG ("grabs : %i", display->xgrabcount);
 }
 
-void 
+void
 myDisplayAddClient (DisplayInfo *display, Client *c)
 {
     g_return_if_fail (c != None);
@@ -410,7 +456,7 @@ myDisplayAddClient (DisplayInfo *display, Client *c)
     display->clients = g_slist_append (display->clients, c);
 }
 
-void 
+void
 myDisplayRemoveClient (DisplayInfo *display, Client *c)
 {
     g_return_if_fail (c != None);
@@ -461,7 +507,7 @@ myDisplayGetClientFromWindow (DisplayInfo *display, Window w, int mode)
     return NULL;
 }
 
-void 
+void
 myDisplayAddScreen (DisplayInfo *display, ScreenInfo *screen)
 {
     g_return_if_fail (screen != NULL);
@@ -471,7 +517,7 @@ myDisplayAddScreen (DisplayInfo *display, ScreenInfo *screen)
     display->nb_screens = display->nb_screens + 1;
 }
 
-void 
+void
 myDisplayRemoveScreen (DisplayInfo *display, ScreenInfo *screen)
 {
     g_return_if_fail (screen != NULL);
@@ -530,18 +576,13 @@ Window
 myDisplayGetRootFromWindow(DisplayInfo *display, Window w)
 {
     XWindowAttributes attributes;
-    gint test;
 
     g_return_val_if_fail (w != None, None);
     g_return_val_if_fail (display != NULL, None);
 
-    gdk_error_trap_push ();
-    XGetWindowAttributes(display->dpy, w, &attributes);
-    test = gdk_error_trap_pop ();
-
-    if (test)
+    if (!XGetWindowAttributes(display->dpy, w, &attributes))
     {
-        TRACE ("myDisplayGetRootFromWindow: no root found for 0x%lx, error code %i", w, test);
+        TRACE ("myDisplayGetRootFromWindow: no root found for 0x%lx", w);
         return None;
     }
     return attributes.root;
@@ -586,7 +627,30 @@ myDisplayGetScreenFromSystray (DisplayInfo *display, Window w)
 
     return NULL;
 }
-#endif
+#endif /* ENABLE_KDE_SYSTRAY_PROXY */
+
+#ifdef HAVE_XSYNC
+Client *
+myDisplayGetClientFromXSyncAlarm (DisplayInfo *display, XSyncAlarm alarm)
+{
+    GSList *index;
+
+    g_return_val_if_fail (alarm != None, NULL);
+    g_return_val_if_fail (display != NULL, NULL);
+    
+    for (index = display->clients; index; index = g_slist_next (index))
+    {
+        Client *c = (Client *) index->data;
+        if (alarm == c->xsync_alarm)
+        {
+            return (c);
+        }
+    }
+    TRACE ("no client found");
+
+    return NULL;
+}
+#endif /* HAVE_XSYNC */
 
 ScreenInfo *
 myDisplayGetDefaultScreen (DisplayInfo *display)
@@ -604,11 +668,11 @@ myDisplayGetDefaultScreen (DisplayInfo *display)
     return NULL;
 }
 
-Time 
+Time
 myDisplayUpdateCurrentTime (DisplayInfo *display, XEvent *ev)
 {
     g_return_val_if_fail (display != NULL, (Time) CurrentTime);
-    
+
     switch (ev->type)
     {
         case KeyPress:
@@ -639,31 +703,46 @@ myDisplayUpdateCurrentTime (DisplayInfo *display, XEvent *ev)
             display->current_time = (Time) ev->xselection.time;
             break;
         default:
+            display->current_time = (Time) CurrentTime;
             break;
     }
     return display->current_time;
 }
 
-Time 
+Time
 myDisplayGetCurrentTime (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, (Time) CurrentTime);
 
+    TRACE ("myDisplayGetCurrentTime gives timestamp=%u", (unsigned int) display->current_time);
     return (Time) display->current_time;
 }
 
-Time          myDisplayGetLastUserTime      (DisplayInfo *);
-void          myDisplaySetLastUserTime      (DisplayInfo *,
-                                             Time);
-Time 
+Time
+myDisplayGetTime (DisplayInfo * display, Time timestamp)
+{
+    Time time;
+
+    time = timestamp;
+    if (time == (Time) CurrentTime)
+    {
+        time = getXServerTime (display);
+    }
+
+    TRACE ("myDisplayGetTime gives timestamp=%u", (unsigned int) time);
+    return (time);
+}
+
+Time
 myDisplayGetLastUserTime (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, (Time) CurrentTime);
 
+    TRACE ("myDisplayGetLastUserTime gives timestamp=%u", (unsigned int) display->last_user_time);
     return (Time) display->last_user_time;
 }
 
-void 
+void
 myDisplaySetLastUserTime (DisplayInfo *display, Time timestamp)
 {
     g_return_if_fail (display != NULL);
@@ -674,20 +753,12 @@ myDisplaySetLastUserTime (DisplayInfo *display, Time timestamp)
     }
 }
 
-static gdouble
-get_time(void)
-{
-    struct timeval timev;
-
-    gettimeofday(&timev, NULL);
-    return (gdouble)timev.tv_sec + (((gdouble)timev.tv_usec) / 1000000);
-}
-
 gboolean
 myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
 {
 #ifdef HAVE_RENDER
-    gdouble t1, t2, dt;
+    GTimeVal t1, t2;
+    gdouble dt;
     Display *dpy;
     Picture picture1, picture2, picture3;
     XRenderPictFormat *format_src, *format_dst;
@@ -729,7 +800,7 @@ myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
     format_src = XRenderFindStandardFormat (dpy, PictStandardA8);
     g_return_val_if_fail (format_src != NULL , FALSE);
 
-    ximage = XGetImage (dpy, 
+    ximage = XGetImage (dpy,
                         DefaultRootWindow(dpy),
                         x, y, w, h,
                         AllPlanes, ZPixmap);
@@ -738,7 +809,7 @@ myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
     rootPixmap = XCreatePixmap (dpy,
                                 DefaultRootWindow(dpy),
                                 w, h, depth);
-    XPutImage (dpy, rootPixmap, 
+    XPutImage (dpy, rootPixmap,
                DefaultGC (dpy, screen_number), ximage,
                0, 0, 0, 0, w, h);
     XDestroyImage (ximage);
@@ -746,7 +817,7 @@ myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
     attrs.override_redirect = TRUE;
     output = XCreateWindow (dpy,
                             DefaultRootWindow(dpy),
-                            x, y, w, h, 
+                            x, y, w, h,
                             0, CopyFromParent, CopyFromParent,
                             (Visual *) CopyFromParent,
                             CWOverrideRedirect, &attrs);
@@ -756,31 +827,31 @@ myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
                                 DefaultRootWindow(dpy),
                                 1, 1, 8);
 
-    t1 = get_time();
-    
+    g_get_current_time (&t1);
+
     pa.repeat = TRUE;
-    picture1 = XRenderCreatePicture (dpy, 
+    picture1 = XRenderCreatePicture (dpy,
                                      rootPixmap,
                                      format_dst, 0, NULL);
-    picture2 = XRenderCreatePicture (dpy, 
+    picture2 = XRenderCreatePicture (dpy,
                                      fillPixmap,
                                      format_src, CPRepeat, &pa);
-    picture3 = XRenderCreatePicture (dpy, 
+    picture3 = XRenderCreatePicture (dpy,
                                      output,
                                      format_dst, 0, NULL);
-    XRenderComposite (dpy, PictOpSrc, 
-                    picture1, None, picture3, 
+    XRenderComposite (dpy, PictOpSrc,
+                    picture1, None, picture3,
                     0, 0, 0, 0, 0, 0, w, h);
     XRenderFillRectangle (dpy, PictOpSrc,
-                          picture2, &c, 0, 0, 
+                          picture2, &c, 0, 0,
                           1, 1);
     for (iterations = 0; iterations < 10; iterations++)
     {
-        XRenderComposite (dpy, PictOpOver, 
-                        picture1, picture2, picture3, 
+        XRenderComposite (dpy, PictOpOver,
+                        picture1, picture2, picture3,
                         0, 0, 0, 0, 0, 0, w, h);
-        ximage = XGetImage (dpy, output, 
-                            0, 0, 1, 1, 
+        ximage = XGetImage (dpy, output,
+                            0, 0, 1, 1,
                             AllPlanes, ZPixmap);
         if (ximage)
         {
@@ -796,8 +867,10 @@ myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
 
     XDestroyWindow (dpy, output);
 
-    t2 = get_time();
-    dt = t2 - t1;
+    g_get_current_time (&t2);
+
+    dt = (gdouble) (t2.tv_sec - t1.tv_sec) * G_USEC_PER_SEC +
+         (gdouble) (t2.tv_usec - t1.tv_usec) / 1000.0;
 
     if (dt < min_time)
     {
