@@ -1,4 +1,4 @@
-/*      $Id: frame.c 23925 2006-11-21 20:47:48Z olivier $
+/*      $Id: frame.c 25371 2007-04-02 19:32:01Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -37,6 +37,10 @@
 #include "frame.h"
 #include "compositor.h"
 
+#ifndef ShapeInput
+#define ShapeInput 2;
+#endif
+
 int
 frameDecorationLeft (ScreenInfo *screen_info)
 {
@@ -807,6 +811,13 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
     XShapeCombineRectangles (display_info->dpy, shape_win, ShapeBounding, 0, 0, &rect, 1, ShapeIntersect, Unsorted);
     XShapeCombineShape (display_info->dpy, c->frame, ShapeBounding, 0, 0, shape_win, ShapeBounding, ShapeSet);
 
+    /* Set Input shape when using XShape extension 1.1 and later */
+    if (display_info->shape_version >= 1001)
+    {
+        XShapeCombineShape (display_info->dpy, shape_win, ShapeInput, frameLeft (c), frameTop (c), c->window, ShapeBounding, ShapeSubtract);
+        XShapeCombineShape (display_info->dpy, shape_win, ShapeInput, frameLeft (c), frameTop (c), c->window, ShapeInput, ShapeUnion);
+        XShapeCombineShape (display_info->dpy, c->frame, ShapeInput, 0, 0, shape_win, ShapeInput, ShapeSet);
+    }
     XDestroyWindow (display_info->dpy, shape_win);
 }
 
@@ -876,6 +887,13 @@ frameDraw (Client * c, gboolean clear_all)
         width_changed = TRUE;
         height_changed = TRUE;
         requires_clearing = TRUE;
+        frameClearQueueDraw (c);
+        if (c->frame_timeout_id)
+        {
+            g_source_remove (c->frame_timeout_id);
+            c->frame_timeout_id = 0;
+        }
+
     }
     else
     {
@@ -1130,3 +1148,48 @@ frameDraw (Client * c, gboolean clear_all)
         frameSetShape (c, 0, NULL, 0);
     }
 }
+
+static gboolean
+update_frame_idle_cb (gpointer data)
+{
+    Client *c;
+    
+    TRACE ("entering update_frame_idle_cb");
+
+    c = (Client *) data;
+    g_return_val_if_fail (c, FALSE);
+
+    frameDraw (c, TRUE);
+    c->frame_timeout_id = 0;
+
+    return FALSE;
+}
+
+void
+frameClearQueueDraw (Client * c)
+{
+    g_return_if_fail (c);
+
+    TRACE ("entering frameClearQueueDraw for \"%s\" (0x%lx)", c->name, c->window);
+
+    if (c->frame_timeout_id)
+    {
+        g_source_remove (c->frame_timeout_id);
+        c->frame_timeout_id = 0;
+    }
+}
+
+void
+frameQueueDraw (Client * c)
+{
+    g_return_if_fail (c);
+
+    TRACE ("entering frameQueueDraw for \"%s\" (0x%lx)", c->name, c->window);
+
+    if (c->frame_timeout_id == 0)
+    {
+        c->frame_timeout_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, 
+                                              update_frame_idle_cb, c, NULL);
+    }
+}
+
