@@ -1,23 +1,23 @@
-/*      $Id: netwm.c 22871 2006-08-24 17:08:39Z olivier $
- 
+/*      $Id: netwm.c 25409 2007-04-06 19:34:10Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -29,12 +29,13 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "client.h"
 #include "compositor.h"
 #include "display.h"
 #include "frame.h"
+#include "focus.h"
 #include "hints.h"
 #include "misc.h"
 #include "netwm.h"
@@ -125,11 +126,11 @@ clientSetNetState (Client * c)
         TRACE ("clientSetNetState : demands_attention");
         data[i++] = display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION];
     }
-    XChangeProperty (display_info->dpy, c->window, 
+    XChangeProperty (display_info->dpy, c->window,
                      display_info->atoms[NET_WM_STATE], XA_ATOM, 32,
                      PropModeReplace, (unsigned char *) data, i);
     /*
-       We also set GNOME hint here for consistency and convenience, 
+       We also set GNOME hint here for consistency and convenience,
        although the meaning of net_wm_state and win_state aren't the same.
      */
     setHint (display_info, c->window, WIN_STATE, c->win_state);
@@ -179,7 +180,7 @@ clientGetNetState (Client * c)
             FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
         }
     }
-    
+
     if (getAtomList (display_info, c->window, NET_WM_STATE, &atoms, &n_atoms))
     {
         int i;
@@ -291,12 +292,12 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
 
-    action = ((XEvent *) ev)->xclient.data.l[0];
-    first  = ((XEvent *) ev)->xclient.data.l[1];
-    second = ((XEvent *) ev)->xclient.data.l[2];
+    action = ev->data.l[0];
+    first  = ev->data.l[1];
+    second = ev->data.l[2];
     mode = 0;
 
-    if ((first  == display_info->atoms[NET_WM_STATE_SHADED]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_SHADED]) ||
         (second == display_info->atoms[NET_WM_STATE_SHADED]))
     {
         if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
@@ -313,7 +314,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_STICKY]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_STICKY]) ||
         (second == display_info->atoms[NET_WM_STATE_STICKY]))
     {
         if (!clientIsValidTransientOrModal (c) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK))
@@ -334,24 +335,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]) || 
-        (second == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]))
-    {
-        if (action == NET_WM_STATE_ADD)
-        {
-            FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-        }
-        else if (action == NET_WM_STATE_REMOVE)
-        {
-            FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-        }
-        else if (action == NET_WM_STATE_TOGGLE)
-        {
-            FLAG_TOGGLE (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-        }
-    }
-
-    if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
         (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
         (first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
         (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
@@ -396,7 +380,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 {
                     mode |= WIN_STATE_MAXIMIZED_HORIZ;
                 }
-                if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) || 
+                if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
                 {
                     mode |= WIN_STATE_MAXIMIZED_VERT;
@@ -406,7 +390,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_MODAL]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_MODAL]) ||
         (second == display_info->atoms[NET_WM_STATE_MODAL]))
     {
         if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_STATE_MODAL))
@@ -453,7 +437,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_ABOVE]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_ABOVE]) ||
         (second == display_info->atoms[NET_WM_STATE_ABOVE]))
     {
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
@@ -476,7 +460,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_BELOW]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_BELOW]) ||
         (second == display_info->atoms[NET_WM_STATE_BELOW]))
     {
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_ABOVE))
@@ -542,11 +526,171 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
         frameDraw (c, TRUE);
     }
+
+    if ((first  == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]) ||
+        (second == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]))
+    {
+        if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
+        {
+            /* Do not apply NET_WM_STATE_DEMANDS_ATTENTION if client is already focused */
+            if (c != clientGetFocusOrPending ())
+            {
+                FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+                clientSetNetState (c);
+            }
+        }
+        else if ((action == NET_WM_STATE_REMOVE) && FLAG_TEST (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
+        {
+            FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+            clientSetNetState (c);
+        }
+        else if (action == NET_WM_STATE_TOGGLE)
+        {
+            /* Do not apply NET_WM_STATE_DEMANDS_ATTENTION if client is already focused */
+            if (c != clientGetFocusOrPending () || !FLAG_TEST (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
+            {
+                FLAG_TOGGLE (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+                clientSetNetState (c);
+            }
+        }
+    }
+}
+
+void
+clientNetMoveResize (Client * c, XClientMessageEvent * ev)
+{
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    unsigned int button_mask;
+    int x_root, y_root, dx, dy, action, button;
+    int corner;
+    gboolean resize; /* true == resize, false == move */
+    XEvent *event;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientNetMoveResize");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+
+    x_root = (int) ev->data.l[0];
+    y_root = (int) ev->data.l[1];
+    action = (int) ev->data.l[2];
+    button = (int) ev->data.l[3];
+    event  = (XEvent *) ev;
+
+    if (button == 0)
+    {
+        button_mask = getMouseXY (screen_info, c->window, &dx, &dy);
+        if (button_mask & Button1Mask)
+        {
+            button = Button1;
+        }
+        else if (button_mask & Button2Mask)
+        {
+            button = Button2;
+        }
+        else if (button_mask & Button3Mask)
+        {
+            button = Button3;
+        }
+        else
+        {
+            g_warning ("Could not determine the mouse button used");
+            return;
+        }
+    }
+
+    corner = CORNER_BOTTOM_RIGHT;
+    resize = TRUE;
+
+    event->xbutton.button = button;
+    event->xbutton.x_root = event->xkey.x_root = x_root;
+    event->xbutton.y_root = event->xkey.y_root = y_root;
+    event->xbutton.time = event->xkey.time = myDisplayGetCurrentTime (display_info);
+
+    switch (action)
+    {
+        /* Keyboard */
+        case NET_WM_MOVERESIZE_SIZE_KEYBOARD:
+            event->type = KeyPress;
+            corner = CORNER_BOTTOM_RIGHT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_MOVE_KEYBOARD:
+            event->type = KeyPress;
+            resize = FALSE; /* Move */
+            break;
+
+        /* Sides */
+        case NET_WM_MOVERESIZE_SIZE_TOP:
+            event->type = ButtonPress;
+            corner = CORNER_COUNT + SIDE_TOP;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_BOTTOM:
+            event->type = ButtonPress;
+            corner = CORNER_COUNT + SIDE_BOTTOM;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_RIGHT:
+            event->type = ButtonPress;
+            corner = CORNER_COUNT + SIDE_RIGHT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_LEFT:
+            event->type = ButtonPress;
+            corner = CORNER_COUNT + SIDE_LEFT;
+            resize = TRUE; /* Resize */
+            break;
+
+        /* Corners */
+        case NET_WM_MOVERESIZE_SIZE_TOPLEFT:
+            event->type = ButtonPress;
+            corner = CORNER_TOP_LEFT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_TOPRIGHT:
+            event->type = ButtonPress;
+            corner = CORNER_TOP_RIGHT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT:
+            event->type = ButtonPress;
+            corner = CORNER_BOTTOM_LEFT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT:
+            event->type = ButtonPress;
+            corner = CORNER_BOTTOM_RIGHT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_MOVE:
+        default:
+            event->type = ButtonPress;
+            resize = FALSE; /* Move */
+            break;
+    }
+    
+    if (!FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+    {
+        if (resize && FLAG_TEST_ALL (c->xfwm_flags, XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_IS_RESIZABLE))
+        {
+            clientResize (c, corner, event);
+        }
+        else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE))
+        {
+            clientMove (c, event);
+        }
+    }
 }
 
 void
 clientUpdateFullscreenState (Client * c)
 {
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     XWindowChanges wc;
     int layer;
 
@@ -554,6 +698,9 @@ clientUpdateFullscreenState (Client * c)
     TRACE ("entering clientUpdateFullscreenState");
     TRACE ("Update fullscreen state for client \"%s\" (0x%lx)", c->name, c->window);
 
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+
     if (FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
         GdkRectangle rect;
@@ -589,15 +736,15 @@ clientUpdateFullscreenState (Client * c)
     clientSetNetState (c);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MANAGED))
     {
-        /* 
+        /*
            For some reason, the configure can generate EnterNotify events
            on lower windows, causing a nasty race cond with apps trying to
            grab focus in focus follow mouse mode. Grab the pointer to
            avoid these effects
          */
-        myScreenGrabPointer (c->screen_info, EnterWindowMask, None, CurrentTime);
+        myScreenGrabPointer (c->screen_info, EnterWindowMask, None, myDisplayGetCurrentTime (display_info));
         clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight, CFG_FORCE_REDRAW);
-        myScreenUngrabPointer (c->screen_info, CurrentTime);
+        myScreenUngrabPointer (c->screen_info);
     }
     else
     {
@@ -607,6 +754,15 @@ clientUpdateFullscreenState (Client * c)
         c->width = wc.width;
     }
     clientSetLayer (c, layer);
+
+    /* Fullscreen has no decoration at all, update NET_FRAME_EXTENTS accordingly */
+    setNetFrameExtents (display_info,
+                        c->window,
+                        frameTop (c),
+                        frameLeft (c),
+                        frameRight (c),
+                        frameBottom (c));
+
 }
 
 void
@@ -650,13 +806,13 @@ clientGetNetWmType (Client * c)
         i = 0;
         while (i < n_atoms)
         {
-            if ((atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DESKTOP]) || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DOCK])    || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_TOOLBAR]) || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_MENU])    || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG])  || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_NORMAL])  || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_UTILITY]) || 
+            if ((atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DESKTOP]) ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DOCK])    ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_TOOLBAR]) ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_MENU])    ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG])  ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_NORMAL])  ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_UTILITY]) ||
                 (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_SPLASH]))
             {
                 c->type_atom = atoms[i];
@@ -787,24 +943,56 @@ clientSetNetClientList (ScreenInfo * screen_info, Atom a, GList * list)
     }
 }
 
-void
+gboolean
+clientValidateNetStrut (Client * c)
+{
+    ScreenInfo *screen_info;
+    gboolean valid;
+    int max_value;
+    int i;
+
+    g_return_val_if_fail (c != NULL, TRUE);
+    TRACE ("entering clientValidateNetStrut for \"%s\" (0x%lx)", c->name, c->window);
+    screen_info = c->screen_info;
+    max_value = MIN (screen_info->width, screen_info->height) / 4;
+    valid = TRUE;
+
+    for (i = 0; i < 4; i++)
+    {
+        if (c->struts[i] > max_value)
+        {
+           g_warning ("Strut value for application window 0x%lx changed from %d to %d", c->window, c->struts[i], max_value);
+           c->struts[i] = max_value;
+           valid = FALSE;
+        }
+    }
+
+    return valid;
+}
+
+gboolean
 clientGetNetStruts (Client * c)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
+    unsigned long old_flags, new_flags;
+    int old_struts[STRUTS_SIZE];
     gulong *struts;
     int nitems;
     int i;
 
-    g_return_if_fail (c != NULL);
+    g_return_val_if_fail (c != NULL, FALSE);
     TRACE ("entering clientGetNetStruts for \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     struts = NULL;
 
-    for (i = 0; i < 12; i++)
+    /* Save old values */
+    old_flags = c->flags & (CLIENT_FLAG_HAS_STRUT | CLIENT_FLAG_HAS_STRUT_PARTIAL);
+    for (i = 0; i < STRUTS_SIZE; i++)
     {
+        old_struts[i] = c->struts[i];
         c->struts[i] = 0;
     }
     FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_STRUT);
@@ -812,24 +1000,32 @@ clientGetNetStruts (Client * c)
 
     if (getCardinalList (display_info, c->window, NET_WM_STRUT_PARTIAL, &struts, &nitems))
     {
-        if (nitems != 12)
+        if (nitems != STRUTS_SIZE)
         {
             if (struts)
             {
                 XFree (struts);
             }
-            return;
+            /* Restore old values */
+            if (old_flags)
+            {
+                FLAG_SET (c->flags, old_flags);
+                for (i = 0; i < STRUTS_SIZE; i++)
+                {
+                    c->struts[i] = old_struts[i];
+                }
+            }
+            return FALSE;
         }
 
         FLAG_SET (c->flags, CLIENT_FLAG_HAS_STRUT);
         FLAG_SET (c->flags, CLIENT_FLAG_HAS_STRUT_PARTIAL);
-        for (i = 0; i < 12; i++)
+        for (i = 0; i < STRUTS_SIZE; i++)
         {
             c->struts[i] = (int) struts[i];
         }
 
         XFree (struts);
-        workspaceUpdateArea (c->screen_info);
     }
     else if (getCardinalList (display_info, c->window, NET_WM_STRUT, &struts, &nitems))
     {
@@ -839,7 +1035,16 @@ clientGetNetStruts (Client * c)
             {
                 XFree (struts);
             }
-            return;
+            /* Restore old values */
+            if (old_flags)
+            {
+                FLAG_SET (c->flags, old_flags);
+                for (i = 0; i < STRUTS_SIZE; i++)
+                {
+                    c->struts[i] = old_struts[i];
+                }
+            }
+            return FALSE;
         }
 
         FLAG_SET (c->flags, CLIENT_FLAG_HAS_STRUT);
@@ -847,21 +1052,46 @@ clientGetNetStruts (Client * c)
         {
             c->struts[i] = (int) struts[i];
         }
-        for (i = 4; i < 12; i++)
+        for (i = 4; i < STRUTS_SIZE; i++)
         {
             c->struts[i] = 0;
         }
         /* Fill(in values as for partial struts */
-        c->struts[TOP_START_X] = c->struts[BOTTOM_START_X] = 0;
-        c->struts[TOP_END_X] = c->struts[BOTTOM_END_X] = 
+        c->struts[STRUTS_TOP_START_X] = c->struts[STRUTS_BOTTOM_START_X] = 0;
+        c->struts[STRUTS_TOP_END_X] = c->struts[STRUTS_BOTTOM_END_X] =
             c->screen_info->width;
-        c->struts[LEFT_START_Y] = c->struts[RIGHT_START_Y] = 0;
-        c->struts[LEFT_END_Y] = c->struts[RIGHT_END_Y] = 
+        c->struts[STRUTS_LEFT_START_Y] = c->struts[STRUTS_RIGHT_START_Y] = 0;
+        c->struts[STRUTS_LEFT_END_Y] = c->struts[STRUTS_RIGHT_END_Y] =
             c->screen_info->height;
-        
+
         XFree (struts);
-        workspaceUpdateArea (c->screen_info);
     }
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STRUT))
+    {
+        clientValidateNetStrut (c);
+    }
+
+    /* check for a change in struts flags */
+    new_flags = c->flags & (CLIENT_FLAG_HAS_STRUT | CLIENT_FLAG_HAS_STRUT_PARTIAL);
+    if (old_flags != new_flags)
+    {
+        return TRUE;
+    }
+
+    /* Flags haven't changed, check values */
+    if (new_flags)
+    {
+        for (i = 0; i < STRUTS_SIZE; i++)
+        {
+            if (old_struts[i] != c->struts[i])
+            {
+                return TRUE;
+            }
+        }
+    }
+
+    return FALSE;
 }
 
 void
@@ -894,7 +1124,7 @@ clientSetNetActions (Client * c)
     {
         atoms[i++] = display_info->atoms[NET_WM_ACTION_SHADE];
     }
-    XChangeProperty (clientGetXDisplay (c), c->window, display_info->atoms[NET_WM_ALLOWED_ACTIONS], 
+    XChangeProperty (clientGetXDisplay (c), c->window, display_info->atoms[NET_WM_ALLOWED_ACTIONS],
                      XA_ATOM, 32, PropModeReplace, (unsigned char *) atoms, i);
 }
 
@@ -926,8 +1156,8 @@ clientWindowType (Client * c)
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_STICKY |
                 CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
-                XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_HAS_MOVE | 
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE | 
+                XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_HAS_MOVE |
+                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
                 XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_STICK |
                 XFWM_FLAG_HAS_BORDER);
         }
@@ -940,8 +1170,8 @@ clientWindowType (Client * c)
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER |  XFWM_FLAG_HAS_MOVE |
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE | 
-                XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_STICK);
+                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
+                XFWM_FLAG_HAS_MENU);
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_TOOLBAR])
         {
@@ -951,8 +1181,7 @@ clientWindowType (Client * c)
             FLAG_SET (c->flags,
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
-                XFWM_FLAG_HAS_STICK);
+                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE);
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_MENU])
         {
@@ -966,14 +1195,18 @@ clientWindowType (Client * c)
             FLAG_SET (c->flags,
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
-                XFWM_FLAG_HAS_STICK);
+                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE);
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG])
         {
             TRACE ("atom net_wm_window_type_dialog detected");
             c->type = WINDOW_DIALOG;
             c->initial_layer = WIN_LAYER_NORMAL;
+            /* Treat DIALOG without transient_for set as transient for group */
+            if (c->transient_for == None)
+            {
+                c->transient_for = c->screen_info->xroot;
+            }
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_NORMAL])
         {
@@ -988,18 +1221,23 @@ clientWindowType (Client * c)
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             c->type = WINDOW_UTILITY;
             c->initial_layer = WIN_LAYER_NORMAL;
+            /* Treat UTILITY without transient_for set as transient for group */
+            if (c->transient_for == None)
+            {
+                c->transient_for = c->screen_info->xroot;
+            }
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_SPLASH])
         {
             TRACE ("atom net_wm_window_type_splash detected");
             c->type = WINDOW_SPLASHSCREEN;
-            c->initial_layer = WIN_LAYER_ABOVE_DOCK;
+            c->initial_layer = WIN_LAYER_NORMAL;
             FLAG_SET (c->flags,
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER | XFWM_FLAG_HAS_HIDE |
                 XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_MOVE |
-                XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_HAS_STICK);
+                XFWM_FLAG_HAS_RESIZE);
         }
     }
     else
