@@ -1,19 +1,19 @@
-/*      $Id: settings.c 22775 2006-08-15 13:54:19Z olivier $
- 
+/*      $Id: settings.c 25372 2007-04-02 19:48:21Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
         xfwm4    - (c) 2002-2006 Olivier Fourdan
  */
@@ -27,7 +27,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 #include <libxfce4mcs/mcs-client.h>
 
 #include "screen.h"
@@ -55,32 +55,32 @@
 /* Forward static decls. */
 
 static void              check_for_grabs      (ScreenInfo *);
-static void              set_settings_margin  (ScreenInfo *, 
-                                               int , 
+static void              set_settings_margin  (ScreenInfo *,
+                                               int ,
                                                int);
 static void              notify_cb            (const char *,
                                                const char *,
-                                               McsAction, 
+                                               McsAction,
                                                McsSetting *,
                                                void *);
 static GdkFilterReturn   client_event_filter  (GdkXEvent *,
-                                               GdkEvent *, 
+                                               GdkEvent *,
                                                gpointer);
-static void              watch_cb             (Window, 
-                                               Bool, 
-                                               long, 
+static void              watch_cb             (Window,
+                                               Bool,
+                                               long,
                                                void *);
-static void              loadRcData           (ScreenInfo *, 
+static void              loadRcData           (ScreenInfo *,
                                                Settings *);
 static void              loadMcsData          (ScreenInfo *,
                                                Settings *);
-static void              loadTheme            (ScreenInfo *, 
+static void              loadTheme            (ScreenInfo *,
                                                Settings *);
-static gboolean          loadKeyBindings      (ScreenInfo *, 
+static gboolean          loadKeyBindings      (ScreenInfo *,
                                                Settings *);
 static void              unloadTheme          (ScreenInfo *);
 static void              unloadSettings       (ScreenInfo *);
-static gboolean          reloadScreenSettings (ScreenInfo *, 
+static gboolean          reloadScreenSettings (ScreenInfo *,
                                                int);
 
 static void
@@ -103,8 +103,8 @@ set_settings_margin (ScreenInfo *screen_info, int idx, int value)
 
     switch (idx)
     {
-        case LEFT:
-        case RIGHT:
+        case STRUTS_LEFT:
+        case STRUTS_RIGHT:
             if (value < 0)
             {
                 val = 0;
@@ -119,8 +119,8 @@ set_settings_margin (ScreenInfo *screen_info, int idx, int value)
             }
             screen_info->params->xfwm_margins[idx] = val;
             break;
-        case TOP:
-        case BOTTOM:
+        case STRUTS_TOP:
+        case STRUTS_BOTTOM:
             if (value < 0)
             {
                 val = 0;
@@ -156,6 +156,26 @@ set_easy_click (ScreenInfo *screen_info, char *modifier)
     }
 }
 
+static void
+set_activate_action (ScreenInfo *screen_info, const char *value)
+{
+    g_return_if_fail (screen_info != NULL);
+    g_return_if_fail (value != NULL);
+
+    if (!g_ascii_strcasecmp ("bring", value))
+    {
+        screen_info->params->activate_action = ACTIVATE_ACTION_BRING;
+    }
+    else if (!g_ascii_strcasecmp ("switch", value))
+    {
+        screen_info->params->activate_action = ACTIVATE_ACTION_SWITCH;
+    }
+    else
+    {
+        screen_info->params->activate_action = ACTIVATE_ACTION_NONE;
+    }
+}
+
 static void
 notify_cb (const char *name, const char *channel_name, McsAction action, McsSetting * setting, void *data)
 {
@@ -177,7 +197,15 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
             case MCS_ACTION_CHANGED:
                 if (setting->type == MCS_TYPE_INT)
                 {
-                    if (!strcmp (name, "Xfwm/ClickToFocus"))
+                    if (!strcmp (name, "Xfwm/BoxMove"))
+                    {
+                        screen_info->params->box_move = setting->data.v_int;
+                    }
+                    else if (!strcmp (name, "Xfwm/BoxResize"))
+                    {
+                        screen_info->params->box_resize = setting->data.v_int;
+                    }
+                    else if (!strcmp (name, "Xfwm/ClickToFocus"))
                     {
                         screen_info->params->click_to_focus = setting->data.v_int;
                         check_for_grabs (screen_info);
@@ -224,14 +252,6 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     {
                         screen_info->params->wrap_resistance = setting->data.v_int;
                     }
-                    else if (!strcmp (name, "Xfwm/BoxMove"))
-                    {
-                        screen_info->params->box_move = setting->data.v_int;
-                    }
-                    else if (!strcmp (name, "Xfwm/BoxResize"))
-                    {
-                        screen_info->params->box_resize = setting->data.v_int;
-                    }
                 }
                 else if (setting->type == MCS_TYPE_STRING)
                 {
@@ -241,7 +261,7 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     }
                     else if (!strcmp (name, "Xfwm/ThemeName"))
                     {
-                        reloadScreenSettings (screen_info, UPDATE_GRAVITY | UPDATE_CACHE);
+                        reloadScreenSettings (screen_info, UPDATE_MAXIMIZE | UPDATE_GRAVITY | UPDATE_CACHE);
                     }
                     else if (!strcmp (name, "Xfwm/ButtonLayout"))
                     {
@@ -277,19 +297,19 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                 {
                     if (!strcmp (name, "Xfwm/LeftMargin"))
                     {
-                        set_settings_margin (screen_info, LEFT, setting->data.v_int);
+                        set_settings_margin (screen_info, STRUTS_LEFT, setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/RightMargin"))
                     {
-                        set_settings_margin (screen_info, RIGHT, setting->data.v_int);
+                        set_settings_margin (screen_info, STRUTS_RIGHT, setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/BottomMargin"))
                     {
-                        set_settings_margin (screen_info, BOTTOM, setting->data.v_int);
+                        set_settings_margin (screen_info, STRUTS_BOTTOM, setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/TopMargin"))
                     {
-                        set_settings_margin (screen_info, TOP, setting->data.v_int);
+                        set_settings_margin (screen_info, STRUTS_TOP, setting->data.v_int);
                     }
                 }
                 break;
@@ -337,7 +357,7 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                 {
                     if (!strcmp (name, "Xfwm/KeyThemeName"))
                     {
-                        reloadScreenSettings (screen_info, UPDATE_KEY_GRABS);
+                        reloadScreenSettings (screen_info, NO_UPDATE_FLAG);
                     }
                 }
                 break;
@@ -359,7 +379,12 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
             case MCS_ACTION_CHANGED:
                 if (setting->type == MCS_TYPE_INT)
                 {
-                    if (!strcmp (name, "Xfwm/CycleMinimum"))
+                    if (!strcmp (name, "Xfwm/BorderlessMaximize"))
+                    {
+                        screen_info->params->borderless_maximize = setting->data.v_int;
+                        reloadScreenSettings (screen_info, UPDATE_MAXIMIZE);
+                    }
+                    else if (!strcmp (name, "Xfwm/CycleMinimum"))
                     {
                         screen_info->params->cycle_minimum = setting->data.v_int;
                     }
@@ -446,7 +471,7 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     else if (!strcmp (name, "Xfwm/UseCompositing"))
                     {
                         screen_info->params->use_compositing = setting->data.v_int;
-                        compositorActivateScreen (screen_info, 
+                        compositorActivateScreen (screen_info,
                                                   screen_info->params->use_compositing);
                     }
                     else if (!strcmp (name, "Xfwm/WrapLayout"))
@@ -464,6 +489,10 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     {
                         reloadScreenSettings (screen_info, UPDATE_BUTTON_GRABS);
                     }
+                    else if (!strcmp (name, "Xfwm/ActivateAction"))
+                    {
+                        set_activate_action (screen_info, setting->data.v_string);
+                    }
                 }
                 break;
             case MCS_ACTION_DELETED:
@@ -536,7 +565,7 @@ loadRcData (ScreenInfo *screen_info, Settings *rc)
     {
         system_keytheme = getSystemThemeDir ();
         parseRc (KEYTHEMERC, system_keytheme, rc);
-        
+
         keytheme = getThemeDir (keythemevalue, KEYTHEMERC);
         if (keytheme)
         {
@@ -546,7 +575,7 @@ loadRcData (ScreenInfo *screen_info, Settings *rc)
         }
         g_free (system_keytheme);
     }
-    homedir = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, 
+    homedir = xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
                                            "xfce4" G_DIR_SEPARATOR_S "xfwm4",
                                            FALSE);
     parseRc ("xfwm4rc", homedir, rc);
@@ -715,6 +744,18 @@ loadMcsData (ScreenInfo *screen_info, Settings *rc)
         }
 
         /* Tweaks channel */
+        if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/ActivateAction", CHANNEL5,
+                &setting) == MCS_SUCCESS)
+        {
+            setValue ("activate_action", setting->data.v_string, rc);
+            mcs_setting_free (setting);
+        }
+        if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/BorderlessMaximize", CHANNEL5,
+                &setting) == MCS_SUCCESS)
+        {
+            setBooleanValueFromInt ("borderless_maximize", setting->data.v_int, rc);
+            mcs_setting_free (setting);
+        }
         if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/CycleMinimum", CHANNEL5,
                 &setting) == MCS_SUCCESS)
         {
@@ -902,7 +943,7 @@ static int
 getTitleShadow (Settings *rc, const gchar * name)
 {
     const gchar *val;
-    
+
     val = getValue (name, rc);
     if (!g_ascii_strcasecmp ("true", val) || !g_ascii_strcasecmp ("under", val))
     {
@@ -959,13 +1000,13 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     theme = getThemeDir (getValue ("theme", rc), THEMERC);
     parseRc (THEMERC, theme, rc);
 
-    screen_info->params->shadow_delta_x = 
+    screen_info->params->shadow_delta_x =
         - (TOINT (getValue ("shadow_delta_x", rc)));
-    screen_info->params->shadow_delta_y = 
+    screen_info->params->shadow_delta_y =
         - (TOINT (getValue ("shadow_delta_y", rc)));
-    screen_info->params->shadow_delta_width = 
+    screen_info->params->shadow_delta_width =
         - (TOINT (getValue ("shadow_delta_width", rc)));
-    screen_info->params->shadow_delta_height = 
+    screen_info->params->shadow_delta_height =
         - (TOINT (getValue ("shadow_delta_height", rc)));
 
     for (i = 0; i < XPM_COLOR_SYMBOL_SIZE; i++)
@@ -993,12 +1034,12 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
             pango_font_description_free (desc);
         }
     }
- 
+
     setXfwmColor (screen_info, &screen_info->title_colors[ACTIVE], rc, 0, "fg", "selected");
     setXfwmColor (screen_info, &screen_info->title_colors[INACTIVE], rc, 1, "fg", "insensitive");
     setXfwmColor (screen_info, &screen_info->title_shadow_colors[ACTIVE], rc, 2, "dark", "selected");
     setXfwmColor (screen_info, &screen_info->title_shadow_colors[INACTIVE], rc, 3, "dark", "insensitive");
-    
+
     if (screen_info->black_gc)
     {
         g_object_unref (G_OBJECT (screen_info->black_gc));
@@ -1136,7 +1177,7 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     screen_info->params->title_shadow[ACTIVE] = getTitleShadow (rc, "title_shadow_active");
     screen_info->params->title_shadow[INACTIVE] = getTitleShadow (rc, "title_shadow_inactive");
 
-    strncpy (screen_info->params->button_layout, getValue ("button_layout", rc), 7);
+    strncpy (screen_info->params->button_layout, getValue ("button_layout", rc), BUTTON_COUNT + 1);
     screen_info->params->button_spacing = TOINT (getValue ("button_spacing", rc));
     screen_info->params->button_offset = TOINT (getValue ("button_offset", rc));
     screen_info->params->maximized_offset = TOINT (getValue ("maximized_offset", rc));
@@ -1158,8 +1199,8 @@ loadKeyBindings (ScreenInfo *screen_info, Settings *rc)
     gchar *keythemevalue;
 
     dpy = myScreenGetXDisplay (screen_info);
-    /* 
-       Load defaults keytheme so that even if there are 
+    /*
+       Load defaults keytheme so that even if there are
        missing shortcuts in an older user defined key theme
        the missing keys will be taken from the default
      */
@@ -1185,6 +1226,7 @@ loadKeyBindings (ScreenInfo *screen_info, Settings *rc)
     }
 
     parseKeyString (dpy, &screen_info->params->keys[KEY_ADD_WORKSPACE], getValue ("add_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_CANCEL], getValue ("cancel_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_CLOSE_WINDOW], getValue ("close_window_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_CYCLE_WINDOWS], getValue ("cycle_windows_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_DEL_WORKSPACE], getValue ("del_workspace_key", rc));
@@ -1195,7 +1237,6 @@ loadKeyBindings (ScreenInfo *screen_info, Settings *rc)
     parseKeyString (dpy, &screen_info->params->keys[KEY_MAXIMIZE_HORIZ], getValue ("maximize_horiz_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_MAXIMIZE_VERT], getValue ("maximize_vert_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_MAXIMIZE_WINDOW], getValue ("maximize_window_key", rc));
-    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_CANCEL], getValue ("move_resize_cancel_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_DOWN], getValue ("move_window_down_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_DOWN_WORKSPACE], getValue ("move_window_down_workspace_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_LEFT], getValue ("move_window_left_key", rc));
@@ -1219,6 +1260,7 @@ loadKeyBindings (ScreenInfo *screen_info, Settings *rc)
     parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_WORKSPACE_11], getValue ("move_window_workspace_11_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_WORKSPACE_12], getValue ("move_window_workspace_12_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_NEXT_WORKSPACE], getValue ("next_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_POPUP_MENU], getValue ("popup_menu_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_PREV_WORKSPACE], getValue ("prev_workspace_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_RAISE_WINDOW], getValue ("raise_window_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_RESIZE_DOWN], getValue ("resize_window_down_key", rc));
@@ -1244,6 +1286,7 @@ loadKeyBindings (ScreenInfo *screen_info, Settings *rc)
     parseKeyString (dpy, &screen_info->params->keys[KEY_WORKSPACE_11], getValue ("workspace_11_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_WORKSPACE_12], getValue ("workspace_12_key", rc));
 
+    myScreenUngrabKeys (screen_info);
     myScreenGrabKeys (screen_info);
 
     return TRUE;
@@ -1252,6 +1295,7 @@ loadKeyBindings (ScreenInfo *screen_info, Settings *rc)
 gboolean
 loadSettings (ScreenInfo *screen_info)
 {
+    const gchar *value;
     Settings rc[] = {
         /* Do not change the order of the following parameters */
         {"active_text_color", NULL, FALSE},
@@ -1277,6 +1321,8 @@ loadSettings (ScreenInfo *screen_info)
         {"inactive_shadow_2", NULL, FALSE},
         {"inactive_mid_2", NULL, FALSE},
         /* You can change the order of the following parameters */
+        {"activate_action", NULL, TRUE},
+        {"borderless_maximize", NULL, TRUE},
         {"box_move", NULL, TRUE},
         {"box_resize", NULL, TRUE},
         {"button_layout", NULL, TRUE},
@@ -1341,6 +1387,7 @@ loadSettings (ScreenInfo *screen_info)
         {"wrap_workspaces", NULL, TRUE},
         /* Keys */
         {"add_workspace_key", NULL, TRUE},
+        {"cancel_key", NULL, TRUE},
         {"close_window_key", NULL, TRUE},
         {"cycle_windows_key", NULL, TRUE},
         {"del_workspace_key", NULL, TRUE},
@@ -1352,13 +1399,13 @@ loadSettings (ScreenInfo *screen_info)
         {"maximize_horiz_key", NULL, TRUE},
         {"maximize_vert_key", NULL, TRUE},
         {"maximize_window_key", NULL, TRUE},
-        {"move_resize_cancel_key", NULL, TRUE},
         {"move_window_down_key", NULL, TRUE},
         {"move_window_down_workspace_key", NULL, TRUE},
         {"move_window_left_key", NULL, TRUE},
         {"move_window_left_workspace_key", NULL, TRUE},
         {"move_window_next_workspace_key", NULL, TRUE},
         {"move_window_prev_workspace_key", NULL, TRUE},
+        {"popup_menu_key", NULL, TRUE},
         {"move_window_right_key", NULL, TRUE},
         {"move_window_right_workspace_key", NULL, TRUE},
         {"move_window_up_key", NULL, TRUE},
@@ -1414,10 +1461,12 @@ loadSettings (ScreenInfo *screen_info)
         return FALSE;
     }
 
+    screen_info->params->borderless_maximize =
+        !g_ascii_strcasecmp ("true", getValue ("borderless_maximize", rc));
     screen_info->params->box_resize =
         !g_ascii_strcasecmp ("true", getValue ("box_resize", rc));
-    screen_info->params->box_move = !g_ascii_strcasecmp ("true", getValue ("box_move", rc));
-
+    screen_info->params->box_move =
+        !g_ascii_strcasecmp ("true", getValue ("box_move", rc));
     screen_info->params->click_to_focus =
         !g_ascii_strcasecmp ("true", getValue ("click_to_focus", rc));
     screen_info->params->cycle_minimum =
@@ -1434,28 +1483,29 @@ loadSettings (ScreenInfo *screen_info)
         !g_ascii_strcasecmp ("true", getValue ("raise_on_focus", rc));
     screen_info->params->prevent_focus_stealing =
         !g_ascii_strcasecmp ("true", getValue ("prevent_focus_stealing", rc));
-    screen_info->params->raise_delay = abs (TOINT (getValue ("raise_delay", rc)));
+    screen_info->params->raise_delay =
+        abs (TOINT (getValue ("raise_delay", rc)));
     screen_info->params->raise_on_click =
         !g_ascii_strcasecmp ("true", getValue ("raise_on_click", rc));
     screen_info->params->raise_with_any_button =
         !g_ascii_strcasecmp ("true", getValue ("raise_with_any_button", rc));
     screen_info->params->restore_on_move =
         !g_ascii_strcasecmp ("true", getValue ("restore_on_move", rc));
-    screen_info->params->frame_opacity = 
+    screen_info->params->frame_opacity =
         abs (TOINT (getValue ("frame_opacity", rc)));
-    screen_info->params->inactive_opacity = 
+    screen_info->params->inactive_opacity =
         abs (TOINT (getValue ("inactive_opacity", rc)));
-    screen_info->params->move_opacity = 
+    screen_info->params->move_opacity =
         abs (TOINT (getValue ("move_opacity", rc)));
-    screen_info->params->resize_opacity = 
+    screen_info->params->resize_opacity =
         abs (TOINT (getValue ("resize_opacity", rc)));
-    screen_info->params->popup_opacity = 
+    screen_info->params->popup_opacity =
         abs (TOINT (getValue ("popup_opacity", rc)));
-    screen_info->params->placement_ratio = 
+    screen_info->params->placement_ratio =
         abs (TOINT (getValue ("placement_ratio", rc)));
-    screen_info->params->show_app_icon = 
+    screen_info->params->show_app_icon =
         !g_ascii_strcasecmp ("true", getValue ("show_app_icon", rc));
-    screen_info->params->show_frame_shadow = 
+    screen_info->params->show_frame_shadow =
         !g_ascii_strcasecmp ("true", getValue ("show_frame_shadow", rc));
     screen_info->params->show_popup_shadow =
         !g_ascii_strcasecmp ("true", getValue ("show_popup_shadow", rc));
@@ -1465,30 +1515,35 @@ loadSettings (ScreenInfo *screen_info)
         !g_ascii_strcasecmp ("true", getValue ("snap_to_windows", rc));
     screen_info->params->snap_resist =
         !g_ascii_strcasecmp ("true", getValue ("snap_resist", rc));
-    screen_info->params->snap_width = abs (TOINT (getValue ("snap_width", rc)));
+    screen_info->params->snap_width =
+        abs (TOINT (getValue ("snap_width", rc)));
 
-    set_settings_margin (screen_info, LEFT,   TOINT (getValue ("margin_left", rc)));
-    set_settings_margin (screen_info, RIGHT,  TOINT (getValue ("margin_right", rc)));
-    set_settings_margin (screen_info, BOTTOM, TOINT (getValue ("margin_bottom", rc)));
-    set_settings_margin (screen_info, TOP,    TOINT (getValue ("margin_top", rc)));
+    set_settings_margin (screen_info, STRUTS_LEFT,   TOINT (getValue ("margin_left", rc)));
+    set_settings_margin (screen_info, STRUTS_RIGHT,  TOINT (getValue ("margin_right", rc)));
+    set_settings_margin (screen_info, STRUTS_BOTTOM, TOINT (getValue ("margin_bottom", rc)));
+    set_settings_margin (screen_info, STRUTS_TOP,    TOINT (getValue ("margin_top", rc)));
 
     set_easy_click (screen_info, getValue ("easy_click", rc));
+    
+    value = getValue ("activate_action", rc);
+    set_activate_action (screen_info, value);
 
-    if (!g_ascii_strcasecmp ("shade", getValue ("double_click_action", rc)))
+    value = getValue ("double_click_action", rc);
+    if (!g_ascii_strcasecmp ("shade", value))
     {
-        screen_info->params->double_click_action = ACTION_SHADE;
+        screen_info->params->double_click_action = DBL_CLICK_ACTION_SHADE;
     }
-    else if (!g_ascii_strcasecmp ("hide", getValue ("double_click_action", rc)))
+    else if (!g_ascii_strcasecmp ("hide", value))
     {
-        screen_info->params->double_click_action = ACTION_HIDE;
+        screen_info->params->double_click_action = DBL_CLICK_ACTION_HIDE;
     }
-    else if (!g_ascii_strcasecmp ("maximize", getValue ("double_click_action", rc)))
+    else if (!g_ascii_strcasecmp ("maximize", value))
     {
-        screen_info->params->double_click_action = ACTION_MAXIMIZE;
+        screen_info->params->double_click_action = DBL_CLICK_ACTION_MAXIMIZE;
     }
     else
     {
-        screen_info->params->double_click_action = ACTION_NONE;
+        screen_info->params->double_click_action = DBL_CLICK_ACTION_NONE;
     }
 
     if (screen_info->workspace_count < 0)
@@ -1518,7 +1573,8 @@ loadSettings (ScreenInfo *screen_info)
         !g_ascii_strcasecmp ("true", getValue ("wrap_cycle", rc));
     screen_info->params->scroll_workspaces =
         !g_ascii_strcasecmp ("true", getValue ("scroll_workspaces", rc));
-    screen_info->params->wrap_resistance = abs (TOINT (getValue ("wrap_resistance", rc)));
+    screen_info->params->wrap_resistance =
+        abs (TOINT (getValue ("wrap_resistance", rc)));
 
     freeRc (rc);
     return TRUE;
@@ -1541,7 +1597,7 @@ unloadTheme (ScreenInfo *screen_info)
         xfwmPixmapFree (&screen_info->corners[i][ACTIVE]);
         xfwmPixmapFree (&screen_info->corners[i][INACTIVE]);
     }
-    for (i = 0; i < BUTTON_COUNT; i++)
+    for (i = 0; i < BUTTON_LAST; i++)
     {
         xfwmPixmapFree (&screen_info->buttons[i][ACTIVE]);
         xfwmPixmapFree (&screen_info->buttons[i][INACTIVE]);
@@ -1566,7 +1622,7 @@ static void
 unloadSettings (ScreenInfo *screen_info)
 {
     g_return_if_fail (screen_info);
-    
+
     TRACE ("entering unloadSettings");
 
     unloadTheme (screen_info);
@@ -1600,9 +1656,9 @@ reloadSettings (DisplayInfo *display_info, int mask)
     GSList *screens;
 
     g_return_val_if_fail (display_info, FALSE);
-    
+
     TRACE ("entering reloadSettings");
-    
+
     /* Refresh all screens, not just one */
     for (screens = display_info->screens; screens; screens = g_slist_next (screens))
     {
@@ -1625,14 +1681,14 @@ initSettings (ScreenInfo *screen_info)
     int i;
 
     g_return_val_if_fail (screen_info, FALSE);
-    
+
     TRACE ("entering initSettings");
 
     display_info = screen_info->display_info;
     names = NULL;
     val = 0;
     i = 0;
-    
+
     if (!mcs_client_check_manager (myScreenGetXDisplay (screen_info), screen_info->screen, "xfce-mcs-manager"))
     {
         g_warning ("MCS manager not running, startup delayed for 5 seconds");
@@ -1688,7 +1744,7 @@ void
 closeSettings (ScreenInfo *screen_info)
 {
     g_return_if_fail (screen_info);
-    
+
     if (screen_info->mcs_client)
     {
         mcs_client_destroy (screen_info->mcs_client);
