@@ -1,22 +1,22 @@
-/*      $Id: spinning_cursor.h,v 1.12 2004/08/22 22:18:50 olivier Exp $
- 
+/*      $Id: spinning_cursor.h 23704 2006-11-04 22:36:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         mozilla  - (c) 1998 mozilla.org
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +28,6 @@
 
 #include <X11/Xlib.h>
 
-Cursor cursorCreateSpinning (Display * dpy);
+Cursor                   cursorCreateSpinning                   (Display *);
 
 #endif /* INC_SPINNING_CURSOR_H */
