@@ -1,28 +1,28 @@
-/*      $Id: stacking.c 22952 2006-08-30 16:44:24Z olivier $
- 
+/*      $Id: stacking.c 25175 2007-03-15 22:29:49Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <glib.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "display.h"
 #include "screen.h"
@@ -154,28 +154,29 @@ gboolean
 clientIsTopMost (Client *c)
 {
     ScreenInfo *screen_info;
-    GList *index;
-    
+    GList *index, *index2;
+    Client *c2;
+
     g_return_val_if_fail (c != NULL, FALSE);
     TRACE ("entering clientIsTopMost");
-    
+
     screen_info = c->screen_info;
-    
+
     index = g_list_find (screen_info->windows_stack, (gconstpointer) c);
     if (index)
     {
-        GList *index2 = g_list_next (index);
-        if (index2)
+        index2 = g_list_next (index);
+        while (index2)
         {
-            Client *c2 = (Client *) index2->data;
-            return (c2->win_layer > c->win_layer);
-        }
-        else
-        {
-            return TRUE;
+            c2 = (Client *) index2->data;
+            if (FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE) && (c2->win_layer == c->win_layer))
+            {
+                return FALSE;
+            }
+            index2 = g_list_next (index2);
         }
     }
-    return FALSE;
+    return TRUE;
 }
 
 Client *
@@ -255,7 +256,7 @@ clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
         if ((frameX (c2) < x) && (frameX (c2) + frameWidth (c2) > x)
             && (frameY (c2) < y) && (frameY (c2) + frameHeight (c2) > y))
         {
-            if (clientSelectMask (c2, 0, WINDOW_REGULAR_FOCUSABLE) 
+            if (clientSelectMask (c2, INCLUDE_SKIP_PAGER | INCLUDE_SKIP_TASKBAR, WINDOW_REGULAR_FOCUSABLE)
                 && (c2 != exclude))
             {
                 c = c2;
@@ -295,11 +296,11 @@ clientRaise (Client * c, Window wsibling)
     }
     TRACE ("raising client \"%s\" (0x%lx) over (0x%lx)", c->name, c->window, wsibling);
 
-    /* 
+    /*
      * If the raised window is the one that has focus, fine, we can
      * release the grab we have on it since there is no use for it
      * anymore.
-     * 
+     *
      * However, if the raised window is not the focused one, then we
      * end up with some kind of indermination, so we need to regrab
      * the buttons for the user to be able to raise or focus the window
@@ -493,7 +494,7 @@ clientLower (Client * c, Window wsibling)
                 }
             }
         }
-        if ((!client_sibling) || 
+        if ((!client_sibling) ||
             (client_sibling && (client_sibling->win_layer < c->win_layer)))
         {
             client_sibling = clientGetBottomMost (screen_info, c->win_layer, c);
