@@ -1,35 +1,37 @@
-/*      $Id: workspaces.c 22939 2006-08-29 21:57:14Z olivier $
- 
+/*      $Id: workspaces.c 25175 2007-03-15 22:29:49Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
+
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "display.h"
 #include "screen.h"
@@ -150,21 +152,21 @@ modify_with_wrap (int value, int by, int limit, gboolean wrap)
     if (value >= limit)
     {
         if (!wrap)
-        { 
+        {
             value = limit - 1;
         }
-        else 
+        else
         {
             value = value % limit;
         }
     }
     else if (value < 0)
     {
-        if (!wrap) 
+        if (!wrap)
         {
             value = 0;
         }
-        else 
+        else
         {
             value = (value + limit) % limit;
         }
@@ -174,16 +176,16 @@ modify_with_wrap (int value, int by, int limit, gboolean wrap)
 
 /* returns TRUE if the workspace was changed, FALSE otherwise */
 gboolean
-workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c)
+workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c, Time timestamp)
 {
     int row, col, newrow, newcol, previous_ws, n;
 
-    workspaceGetPosition(screen_info, screen_info->current_ws, &row, &col);
-    newrow = modify_with_wrap(row, rowmod, screen_info->desktop_layout.rows, screen_info->params->wrap_layout);
-    newcol = modify_with_wrap(col, colmod, screen_info->desktop_layout.cols, screen_info->params->wrap_layout);
-    n = workspaceGetNumber(screen_info, newrow, newcol);
+    workspaceGetPosition (screen_info, screen_info->current_ws, &row, &col);
+    newrow = modify_with_wrap (row, rowmod, screen_info->desktop_layout.rows, screen_info->params->wrap_layout);
+    newcol = modify_with_wrap (col, colmod, screen_info->desktop_layout.cols, screen_info->params->wrap_layout);
+    n = workspaceGetNumber (screen_info, newrow, newcol);
 
-    if (n == screen_info->current_ws) 
+    if (n == screen_info->current_ws)
     {
         return FALSE;
     }
@@ -191,7 +193,7 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c)
     previous_ws = screen_info->current_ws;
     if ((n >= 0) && (n < screen_info->workspace_count))
     {
-        workspaceSwitch(screen_info, n, c, TRUE);
+        workspaceSwitch (screen_info, n, c, TRUE, timestamp);
     }
     else if (screen_info->params->wrap_layout)
     {
@@ -201,33 +203,33 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c)
         }
         else
         {
-            if (colmod > 0) 
+            if (colmod > 0)
             {
                 newcol = 0;
             }
-            else if (rowmod > 0) 
+            else if (rowmod > 0)
             {
                 newrow = 0;
             }
-            else if (rowmod < 0) 
+            else if (rowmod < 0)
             {
                 newrow--;
             }
-            else 
+            else
             {
                 return FALSE;
             }
 
-            n = workspaceGetNumber(screen_info, newrow, newcol);
+            n = workspaceGetNumber (screen_info, newrow, newcol);
         }
-        workspaceSwitch(screen_info, n, c, TRUE);
+        workspaceSwitch (screen_info, n, c, TRUE, timestamp);
     }
 
     return (screen_info->current_ws != previous_ws);
 }
 
 void
-workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean update_focus)
+workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean update_focus, Time timestamp)
 {
     DisplayInfo *display_info;
     Client *c, *new_focus;
@@ -238,11 +240,10 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
     int rx, ry, wx, wy;
     unsigned int mask;
     unsigned long data[1];
-    
+
     TRACE ("entering workspaceSwitch");
 
     display_info = screen_info->display_info;
-    new_focus = NULL;
     if ((new_ws == screen_info->current_ws) && (screen_info->params->toggle_workspaces))
     {
         new_ws = screen_info->previous_ws;
@@ -269,75 +270,90 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
         return;
     }
 
-    myScreenGrabPointer (screen_info, EnterWindowMask, None, myDisplayGetCurrentTime (display_info));
+    myScreenGrabPointer (screen_info, EnterWindowMask, None, timestamp);
 
     screen_info->previous_ws = screen_info->current_ws;
     screen_info->current_ws = new_ws;
+
+    new_focus = NULL;
+    list_hide = NULL;
+    previous  = NULL;
+    c = clientGetFocus ();
+
     if (c2)
     {
         clientSetWorkspace (c2, new_ws, FALSE);
     }
 
-    list_hide = NULL;
-    previous = clientGetFocus ();
+    if (c)
+    {
+        if (c->type & WINDOW_REGULAR_FOCUSABLE)
+        {
+            previous = c;
+        }
+        if (c2 == c)
+        {
+            new_focus = c2;
+        }
+    }
 
-    /* First pass */
+    /* First pass: Show, from top to bottom */
+    for (index = g_list_last(screen_info->windows_stack); index; index = g_list_previous (index))
+    {
+        c = (Client *) index->data;
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
+        {
+            clientSetWorkspace (c, new_ws, TRUE);
+        }
+        else if (c->win_workspace == new_ws)
+        {
+            if (!FLAG_TEST (c->flags, CLIENT_FLAG_ICONIFIED) && !FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
+            {
+                if (!clientIsTransientOrModal (c) || !clientTransientOrModalHasAncestor (c, new_ws))
+                {
+                    clientShow (c, FALSE);
+                }
+            }
+        }
+    }
+
+    /* Second pass: Hide from bottom to top */
     for (index = screen_info->windows_stack; index; index = g_list_next (index))
     {
         c = (Client *) index->data;
-        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE)
-            && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY) 
-            && ((c->win_workspace != new_ws)))
+        
+        if (c->win_workspace != new_ws)
         {
             if (c == previous)
             {
                 FLAG_SET (previous->xfwm_flags, XFWM_FLAG_FOCUS);
-                clientSetFocus (screen_info, NULL, myDisplayGetCurrentTime (display_info), FOCUS_IGNORE_MODAL);
+                clientSetFocus (screen_info, NULL, timestamp, FOCUS_IGNORE_MODAL);
             }
-            if (!clientIsValidTransientOrModal (c))
+            if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE) && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
             {
-                /* Just build of windows that will be hidden, so that
-                   we can record the previous focus even when on a
-                   transient (otherwise the transient vanishes along
-                   with its parent window which is placed below...
-                 */
-                list_hide = g_list_append (list_hide, c);
+                if (!clientIsTransientOrModal (c) || !clientTransientOrModalHasAncestor (c, new_ws))
+                {
+                    clientHide (c, new_ws, FALSE);
+                }
             }
         }
     }
 
-    /* First pass and a half :) */
-    if (list_hide)
-    {
-        for (index = list_hide; index; index = g_list_next (index))
-        {
-            c = (Client *) index->data;
-            clientHide (c, new_ws, FALSE);
-        }
-        g_list_free (list_hide);
-    }
-
-    /* Second pass */
+    /* Third pass: Check for focus, from top to bottom */
     for (index = g_list_last(screen_info->windows_stack); index; index = g_list_previous (index))
     {
         c = (Client *) index->data;
-        if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY)
-            || FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
+        
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
         {
-            clientSetWorkspace (c, new_ws, TRUE);
-            if (c == previous)
+            if ((!new_focus) && (c == previous) && clientSelectMask (c, 0, WINDOW_REGULAR_FOCUSABLE))
             {
                 new_focus = c;
             }
             FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_FOCUS);
         }
-        else if ((c->win_workspace == new_ws)
-            && !FLAG_TEST (c->flags, CLIENT_FLAG_ICONIFIED))
+        else if (c->win_workspace == new_ws)
         {
-            if (!clientIsValidTransientOrModal (c))
-            {
-                clientShow (c, FALSE);
-            }
             if ((!new_focus) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_FOCUS))
             {
                 new_focus = c;
@@ -348,7 +364,7 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
 
     setHint (display_info, screen_info->xroot, WIN_WORKSPACE, new_ws);
     data[0] = new_ws;
-    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, 
+    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot,
                      display_info->atoms[NET_CURRENT_DESKTOP], XA_CARDINAL, 32,
                      PropModeReplace, (unsigned char *) data, 1);
     if (!(screen_info->params->click_to_focus))
@@ -363,12 +379,26 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
         }
     }
 
-    myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
-
-    if (new_focus && update_focus)
+    if (update_focus)
     {
-        clientSetFocus (new_focus->screen_info, new_focus, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
+        if (new_focus)
+        {
+            if ((screen_info->params->click_to_focus) && (screen_info->params->raise_on_click))
+            {
+                if (!(screen_info->params->raise_on_focus) && !clientIsTopMost (new_focus))
+                {
+                    clientRaise (new_focus, None);
+                }
+            }
+            clientSetFocus (screen_info, new_focus, timestamp, FOCUS_SORT);
+        }
+        else
+        {
+            clientFocusTop (screen_info, WIN_LAYER_NORMAL, timestamp);
+        }
     }
+
+    myScreenUngrabPointer (screen_info);
 }
 
 void
@@ -415,12 +445,12 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
     }
     if (screen_info->current_ws > count - 1)
     {
-        workspaceSwitch (screen_info, count - 1, NULL, TRUE);
+        workspaceSwitch (screen_info, count - 1, NULL, TRUE, myDisplayGetCurrentTime (display_info));
     }
-    setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
+    setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
                     screen_info->width, screen_info->height, screen_info->margins);
     /* Recompute the layout based on the (changed) number of desktops */
-    getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, 
+    getDesktopLayout (display_info, screen_info->xroot, screen_info->workspace_count,
                      &screen_info->desktop_layout);
 }
 
@@ -442,10 +472,10 @@ workspaceUpdateArea (ScreenInfo *screen_info)
     TRACE ("entering workspaceUpdateArea");
 
     display_info = screen_info->display_info;
-    prev_top = screen_info->margins[TOP];
-    prev_left = screen_info->margins[LEFT];
-    prev_right = screen_info->margins[RIGHT];
-    prev_bottom = screen_info->margins[BOTTOM];
+    prev_top = screen_info->margins[STRUTS_TOP];
+    prev_left = screen_info->margins[STRUTS_LEFT];
+    prev_right = screen_info->margins[STRUTS_RIGHT];
+    prev_bottom = screen_info->margins[STRUTS_BOTTOM];
 
     for (i = 0; i < 4; i++)
     {
@@ -457,19 +487,19 @@ workspaceUpdateArea (ScreenInfo *screen_info)
         if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STRUT)
             && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
         {
-            screen_info->margins[TOP]    = MAX (screen_info->margins[TOP],    c->struts[TOP]);
-            screen_info->margins[LEFT]   = MAX (screen_info->margins[LEFT],   c->struts[LEFT]);
-            screen_info->margins[RIGHT]  = MAX (screen_info->margins[RIGHT],  c->struts[RIGHT]);
-            screen_info->margins[BOTTOM] = MAX (screen_info->margins[BOTTOM], c->struts[BOTTOM]);
+            screen_info->margins[STRUTS_TOP]    = MAX (screen_info->margins[STRUTS_TOP],    c->struts[STRUTS_TOP]);
+            screen_info->margins[STRUTS_LEFT]   = MAX (screen_info->margins[STRUTS_LEFT],   c->struts[STRUTS_LEFT]);
+            screen_info->margins[STRUTS_RIGHT]  = MAX (screen_info->margins[STRUTS_RIGHT],  c->struts[STRUTS_RIGHT]);
+            screen_info->margins[STRUTS_BOTTOM] = MAX (screen_info->margins[STRUTS_BOTTOM], c->struts[STRUTS_BOTTOM]);
         }
     }
 
-    if ((prev_top != screen_info->margins[TOP]) || (prev_left != screen_info->margins[LEFT])
-        || (prev_right != screen_info->margins[RIGHT])
-        || (prev_bottom != screen_info->margins[BOTTOM]))
+    if ((prev_top != screen_info->margins[STRUTS_TOP]) || (prev_left != screen_info->margins[STRUTS_LEFT])
+        || (prev_right != screen_info->margins[STRUTS_RIGHT])
+        || (prev_bottom != screen_info->margins[STRUTS_BOTTOM]))
     {
         TRACE ("Margins have changed, updating net_workarea");
-        setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
+        setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
                         screen_info->width, screen_info->height, screen_info->margins);
         /* Also prevent windows from being off screen, just like when screen is resized */
         clientScreenResize(screen_info);
